
#include "grafika.h"
#include "stm32f10x.h"

#include "NokiaLCD.h"

#define INTRO 1
#define GRA 2
#define GAMEOVER 3
#define PAUZA 4

void RCC_Configuration(void);
void GPIO_Configuration(void);
void EXTI_Configuration(void);
void NVIC_Configuration(void);
unsigned char Kolizja(unsigned char x, unsigned char y, unsigned char r);
void KlocekObroc(void);
void KlocekWklej(void);
void KlocekNowy(void);
void KlocekLewo(void);
void KlocekPrawo(void);
void KlocekDol(void);
void RysujEkran(void);
void CzyscPlansze(void);

const unsigned char klocki[7][4][4][2]={
{{{1,1},{1,2},{2,1},{2,2}},
 {{1,1},{1,2},{2,1},{2,2}},
 {{1,1},{1,2},{2,1},{2,2}},
 {{1,1},{1,2},{2,1},{2,2}}
},	  //kwadrat
{{{1,0},{1,1},{2,1},{2,2}},
 {{1,1},{2,1},{0,2},{1,2}},
 {{1,0},{1,1},{2,1},{2,2}},
 {{1,1},{2,1},{0,2},{1,2}}
},	//eska prawa

{{{2,0},{2,1},{1,1},{1,2}},
 {{0,0},{1,0},{1,1},{2,1}},
 {{2,0},{2,1},{1,1},{1,2}},
 {{0,0},{1,0},{1,1},{2,1}}
},	//eska lewa

{{{1,0},{1,1},{1,2},{2,2}},
 {{1,1},{2,1},{3,1},{1,2}},
 {{1,1},{2,1},{2,2},{2,3}},
 {{2,1},{2,2},{1,2},{0,2}}
},  //elka prawa

{{{2,0},{2,1},{2,2},{1,2}},
 {{1,1},{1,2},{2,2},{3,2}},
 {{1,1},{2,1},{1,2},{1,3}},
 {{0,1},{1,1},{2,1},{2,2}}
},	//elka lewa

{{{1,0},{1,1},{1,2},{2,1}},
 {{0,1},{1,1},{2,1},{1,2}},
 {{0,1},{1,0},{1,1},{1,2}},
 {{1,0},{0,1},{1,1},{2,1}}
},  //trojkacik

{{{1,0},{1,1},{1,2},{1,3}},
 {{0,1},{1,1},{2,1},{3,1}},
 {{1,0},{1,1},{1,2},{1,3}},
 {{0,1},{1,1},{2,1},{3,1}}
}  //patyczek
};

//Tablica przechowujca plansz gry
unsigned char plansza[10][20];

//Zmienne pooenia klocka x,y i obrt sowo
unsigned char klocekx, kloceky, klocekr;

//Zmienna okrela rodzaj spadajcego klocka
unsigned char kloceknr;
//Zmienna okrela rodzaj nastpnego klocka
unsigned char nkloceknr;
//Zmienna na potrzeby losowania rodzaju klocka
unsigned char rkloceknr;

//Zmienna okrelajca stan programu
unsigned char stan_gry=INTRO;
//Punkty
unsigned int punkty;
//Ilo zaliczonych linii
unsigned int linie;
//Wpisanie 1 spowoduje odwieenie wywietlacza
unsigned char refresh;

//Prdko gry
unsigned char szybkosc;
//Zmienna odliczajca linie przed zwikszeniem prdkoci gry
unsigned char linie_etap;

//Zmienna odlicza ilo zaliczonych linii do kolejnego zwikszenia prdkoci gry
//Zmienna odlicza czas midzy przesuniciami klocka w d
unsigned char stimer;

//Zmienna okrela przesunicie ta w czasie trwania intra
unsigned char introbgoff=0;

int main(void)
{
	//Konfiguracja systemu taktowania
  RCC_Configuration();
	//Konfiguracja GPIO
  GPIO_Configuration();
	//Konfiguracja przerwa
  EXTI_Configuration();
  NVIC_Configuration();

  //konfiguracja timera SysTick
	#define SysTick_Freq 9000000
	SysTick_Config(SysTick_Freq/10);		//
	SysTick_CLKSourceConfig(SysTick_CLKSource_HCLK_Div8);

	//Inicjalizacja wywietlacza LCD
	LcdInit();

	//Czyszczenie planszy
	CzyscPlansze();
	//Zaadowanie nowego klocka
	KlocekNowy();
	//Zerowanie licznikw z punktami
  punkty=0;
	linie=0;

  szybkosc=10;

  stan_gry=INTRO;

  refresh=1;

  while (1)
  {
    if(refresh==1)
    {
      refresh=0;
      switch(stan_gry) //sprawdzanie stanu gry
      {
      case GRA: //gra uruchomiona
        RysujEkran();
        break;

      case GAMEOVER: //koniec gry
        LcdLoadSubBMP(gameover_tlo); //naoenie biaego ta pod napis GAME OVER
        LcdLoadAddBMP(gameover); //naoenie napisu GAME OVER
        LcdUpdate();
        break;

      case PAUZA: //pauza
      	LcdLoadSubBMP(pauza_tlo); //naoenie biaego ta pod napis PAUSE
        LcdLoadAddBMP(pauza);  //naoenie napisu PAUSE
        LcdUpdate();
        break;

      case INTRO: //intro
      	LcdLoadBG(intro_tlo, intro_tlo_sz, introbgoff); //wywietlenie przesuwajcego si ta intro
      	LcdLoadSubBMP(intro_tlo2); //naoenie biaego ta pod napis PAUSE
        LcdLoadAddBMP(intro);  //naoenie napisu PAUSE
      	LcdUpdate();
      	break;
      }
    }
  }

	return(0);
}

//Funkcja zeruje pami planszy
void CzyscPlansze(void)
{
  unsigned char x,y;

  for(y=0;y<20;y++)				//odliczanie wierszy
  	for(x=0;x<10;x++)			//odliczanie kolumn
	  plansza[x][y] = 0;		//wypenianie komrek zerami
}

//Funkcja wykrywa kolizje klocka z punktami na planszy
//oraz wykroczenie klocka poza plansz
unsigned char Kolizja(unsigned char x, unsigned char y, unsigned char r)
{
  unsigned char kx,ky,i,kolizja;
  kolizja=0;

  for(i=0;i<4;i++)  	//ptla odlicza kolejne cztery segmenty klocka
  {										//(kady klocek skada si z czterech segmentw)
    kx=klocki[kloceknr][r][i][0]+x-2; //obliczanie wsprzdnych na planszy danego segmentu klocka
    ky=klocki[kloceknr][r][i][1]+y-2;
    //if(kx < 0) kol=1;
    if(kx > 9) kolizja=1; 					//sprawdzenie czy segment jest poza ciankami bocznymi planszy
    if(ky > 19) kolizja=1; 					//sprawdzenie czy segment jest poniej dna planszy
    if(plansza[kx][ky]) kolizja=1; 	//sprawdzenie czy segment nie nakada si na istniejcy punkt na planszy
  }
  return kolizja;			//jeeli wykryto kolizj to funkcja zwraca 1 w innym razie 0
}

//Funkcja przesuwa klocek w lewo
void KlocekLewo(void)
{
	if(Kolizja(klocekx-1,kloceky,klocekr) == 0)	//sprawdzenie czy przesunity klocek nie wywoa kolizji
	{												//jeeli nie to:
	  klocekx--;						//zmniejsz zmienn x pooenia klocka
 	}
}

//Funkcja przesuwa klocek w prawo
void KlocekPrawo(void)
{
	if(Kolizja(klocekx+1,kloceky,klocekr) == 0)	//sprawdzenie czy przesunity klocek nie wywoa kolizji
	{												//jeeli nie to:
	  klocekx++;						//zwiksz zmienn x pooenia klocka
 	}
}

//Funkcja realizuje obrt klocka
void KlocekObroc(void)
{
	if(Kolizja(klocekx,kloceky,klocekr+1) == 0)	//sprawdzenie czy obrcony klocek nie wywoa kolizji
	{												//jeeli nie to:
	klocekr++;							//zwiksz zmienn obrotu klocka
	if(klocekr > 3) klocekr = 0;
 	}
}

//Funkcja kopiuje biecy klocek na plansz jako jej stae punkty
void KlocekWklej(void)
{
  unsigned char x,y,i;
  for(i=0;i<4;i++)  //ptla odlicza kolejne 4 segmenty klocka
   {
  	x=klocki[kloceknr][klocekr][i][0] + klocekx-2;  //oblicz pooenie segmentu na planszy
  	y=klocki[kloceknr][klocekr][i][1] + kloceky-2;
  	plansza[x][y]=1; 	//zapal punk na planszy w miejscu segmentu klocka
   }
}

//Funkcja aduje nowy klocek
void KlocekNowy(void)
{
  kloceknr=nkloceknr;				//kopiuj rodzaj klocka jaki by nastpny jako bierzcy
  nkloceknr=rkloceknr;			//kopiuj rodzaj nastpnego klocka z zmiennej losujcej

  if(kloceknr==nkloceknr)		//jeeli nastpny klocek jest taki sam jak bierzcy
  {
    rkloceknr++;						//zwiksz zmienn losujc
    if(rkloceknr>6) rkloceknr=0;
    nkloceknr++;						//zwiksz zmienn z rodzajem kolejnego klocka
    if(nkloceknr>6) nkloceknr=0;
  }
  klocekr=0;								//zerowanie pooenia nowego klocka na planszy
  klocekx=5;
  kloceky=2;

  if(Kolizja(klocekx,kloceky,klocekr) == 0)  //sprawdzanie czy nowy klocek nie powoduje kolizji
  {																//jeeli nie:
    punkty++;											//zwiksz punktacj
  }else														//jeeli nowy klocek wywoa od razy kolizj
  {
    stan_gry=GAMEOVER;						//koniec gry!
  }
}

//Funkcja wykrywa pene linie na planszy i kasuje je zwikszajc punkty
void Redukcja(void)
{
  unsigned char x,y,k,yy;

  for(y=0;y<20;y++) //ptla oblicza kolejne linie planszy
  {
    k=1;
    for(x=0;x<10;x++) if(plansza[x][y] == 0) k=0; 	//ptla sprawdza kady punkt danej linii
    												//jeeli brakuje ktrego klocka to p=0;
    if(k == 1)		//jeeli bieca linia jest pena
    {
      linie++;		//zwiksz punkty
      punkty+=10;

      linie_etap++;									//zwiksz zmienn odliczajc klocki do kolejnego zwikszenia prdkoci gry

      if(linie_etap>10)							//sprawdzenie czy zwikszy prdko gry
      {
        linie_etap=0;
        if(szybkosc>0) szybkosc--;
      }

      for(yy=y;yy>0;yy--)  //ptla odlicza kolejne linie od biecej w gr
      {
        for(x=0;x<10;x++) 										//ptla kopiuje lini yy-1 do linii yy
        	plansza[x][yy]=plansza[x][yy-1];		//spowoduje to skasowanie penej linii i przesunicie wszystkich linii ponad w d
      }
    }
  }
}

//Funkcja przesuwa klocek w d
void KlocekDol(void)
{
	if(Kolizja(klocekx,kloceky+1,klocekr) == 0)	//sprawdzenie czy przesunity klocek nie wywoa kolizji
	{									//jeeli nie to:
	  kloceky++;						//zwiksz zmienn y pooenia klocka
 	}else
	{									//jeeli wykryto kolizj:
	  KlocekWklej();						//skopiuj segmenty klocja na plansz
	  Redukcja();						//sprawd czy nie ma penych linii
	  KlocekNowy();						//zaaduj nowy klocek
	}
}

//Funkcja rysowania ekranu podczas gry

#define POFFX 3		//wsp. X grnego lewego rogu planszy
#define POFFY 3		//wsp. Y grnego lewego rogu planszy
#define NKOFFX 30	//wsp. X grnego lewego rogu miejsca wywietlania nastpnego klocka
#define NKOFFY 31	//wsp. Y grnego lewego rogu miejsca wywietlania nastpnego klocka
#define WOFFX 9		//wsp. X wyniku
#define WOFFY 2		//wsp. Y wyniku
#define LOFFX 9		//wsp. X iloci linii
#define LOFFY 5		//wsp. Y iloci linii

void RysujEkran(void)
{
  unsigned char x,y,i;
  unsigned int val,c,d;

  LcdClear();					//czyszczenie ekranu

  LcdLoadBMP(tlo);			//zaadowanie ta

  //rysowanie staych punktw na planszy
  for(y=0;y<20;y++)				//odliczanie wierszy pamici planszy
  for(x=0;x<10;x++)				//odliczanie kolumn
  {
   if(plansza[x][y])				//jeeli dany punkt na planszy jest zapalony
   {
  	 LcdPixel(POFFX+x*2, POFFY+y*2, PIXEL_ON);		//rysowanie kolejnych 4 pikseli punktu
  	 LcdPixel(POFFX+x*2+1, POFFY+y*2, PIXEL_ON);
  	 LcdPixel(POFFX+x*2, POFFY+y*2+1, PIXEL_ON);
  	 LcdPixel(POFFX+x*2+1, POFFY+y*2+1, PIXEL_ON);
   }
  }

  //rysownie spadajcego klocka
  for(i=0;i<4;i++)																		//odliczanie 4 segmentw klocka
   {
		x=klocki[kloceknr][klocekr][i][0] + klocekx-2;		//obliczanie wsprzdnych na planszy danego segmentu klocka
		y=klocki[kloceknr][klocekr][i][1] + kloceky-2;
		LcdPixel(POFFX+x*2, POFFY+y*2, PIXEL_ON);					//rysowanie kolejnych 4 pikseli segmentu
		LcdPixel(POFFX+x*2+1, POFFY+y*2, PIXEL_ON);
		LcdPixel(POFFX+x*2, POFFY+y*2+1, PIXEL_ON);
		LcdPixel(POFFX+x*2+1, POFFY+y*2+1, PIXEL_ON);
   }

  //rysownie klocka, ktry bdzie nastpny
  for(i=0;i<4;i++)																//odliczanie 4 segmentw klocka
   {
		x=klocki[nkloceknr][0][i][0];									//obliczanie wsprzdnych na ekranie danego segmentu klocka
		y=klocki[nkloceknr][0][i][1];
		LcdPixel(NKOFFX+x*2,NKOFFY+y*2,	PIXEL_ON);		//rysowanie kolejnych 4 pikseli segmentu
		LcdPixel(NKOFFX+x*2+1,NKOFFY+y*2,	PIXEL_ON);
		LcdPixel(NKOFFX+x*2,NKOFFY+y*2+1,	PIXEL_ON);
		LcdPixel(NKOFFX+x*2+1,NKOFFY+y*2+1,	PIXEL_ON);
   }

  //wywietlanie wyniku
  LcdGotoXY(WOFFX,WOFFY);
  val=punkty;
  for(d=10000;d>0;d=d/10)						//odliczanie kolejnych miejsc dziesietnych
  {
		c=val/d;												//dzielenie wyniku na tysice, setki itd.
		val=val-(c*d);									//odejmowanie tysicy, setek, itd. od wyniku,
																		//aby mona byo obliczy kolejne miejsce dziesitne w kolejnym kroku ptli
		c=c+'0';												//konwersja wyniku na cyfr w kodzie ASCII
		LcdChr(c);											//wywietlanie kolejnych cyfr wyniku na ekranie
  }

  //Wywietlanie liczby zaliczonych linii
  LcdGotoXY(LOFFX,LOFFY);
  val=linie;
  for(d=10000;d>0;d=d/10)
  {
		c=val/d;
		val=val-(c*d);
		c=c+'0';
		LcdChr(c);
  }

  LcdUpdate();				//Kopiuj bufor na ekran
}

//Funkcja ustawia zmienn refresh
void OdswierzEkran(void)
{
	refresh=1;
}

//Funkcja do odliczania czasu do nastpnego przesunicia klocka w d
//zmienna speed reguluje ten czas
unsigned char TimerGry(void)
{
	unsigned char next=0;

	stimer++;								//zwiksz zmienn odliczajc czas
	if(stimer>szybkosc)			//jeeli minie kolejny odcinek czasu
	{												//zaleny od zmiennej szybko
	stimer=0;
	next=1;
	}
	return next;						//funkcja zwrci warto 1
}

//Funkcje konfiguracyjne
//Funkcja konfigurujca porty GPIO

void GPIO_Configuration(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;																				//Inicjalizacja struktury konfiguracji GPIO

  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC | RCC_APB2Periph_GPIOE | RCC_APB2Periph_AFIO, ENABLE);	//uruchomienie taktowania dla portu GPIOE

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10 | GPIO_Pin_11;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING  ;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_Init(GPIOC, &GPIO_InitStructure);																			//ustawienie pinw 8, 9, 10, 11 i 12 portu GPIOE jako wej
}

//Funkcja konfigurujca system sygnaw taktowania

void RCC_Configuration(void)
{
  ErrorStatus HSEStartUpStatus;

  RCC_DeInit();																																//reset ustawie RCC
  RCC_HSEConfig(RCC_HSE_ON);																									//wcz HSE

  HSEStartUpStatus = RCC_WaitForHSEStartUp();																	//oczekiwanie na gotowo HSE

  if (HSEStartUpStatus == SUCCESS)
  {
	FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);												//wczenie buforu dla pamici Flash
	FLASH_SetLatency(FLASH_Latency_2);																					//ustawienie opnie dla pamici Flash

	RCC_HCLKConfig(RCC_SYSCLK_Div1);																						//taktowanie rdzenia sygnaem 1:1 (72MHz)
	RCC_PCLK2Config(RCC_HCLK_Div1);																							//taktowanie magistrali APB2 sygnaem 1:1 (72MHz)
	RCC_PCLK1Config(RCC_HCLK_Div2);																							//taktowanie magistrali APB1 sygnaem 1:2 (36MHz)

	RCC_PREDIV2Config(RCC_PREDIV2_Div5 );																				//ustawienie PLL2 1:5*8  (25MHz/5*8=20Mhz)
	RCC_PLL2Config(RCC_PLL2Mul_8);
	RCC_PLL2Cmd(ENABLE);
	while(RCC_GetFlagStatus(RCC_FLAG_PLL2RDY) == RESET) {} 											//oczekiwanie na uruchomienie PLL2

  RCC_PREDIV1Config(RCC_PREDIV1_Source_PLL2,RCC_PREDIV1_Div5);								//ustawienie PLL1 PLL2CLK/5*9 (20MHz/5*9=72MHz)
	RCC_PLLConfig(RCC_PLLSource_PREDIV1, RCC_PLLMul_9);
  RCC_PLLCmd (ENABLE);
  while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET) {} 											//oczekiwanie na uruchomienie PLL1

	RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);																	//ustawienie ptli PLL1 jako rda sygnau zegarowego dla systemu
  while (RCC_GetSYSCLKSource() != 0x08) {}																		//oczekiwanie a ptla PLL1 stanie si rdem sygnay zegarowego dla sytemu
  }
}

//Funkcja konfigurujca GPIO jako rda przerwa (kontroler EXTI)

void EXTI_Configuration(void)
{
  EXTI_InitTypeDef EXTI_InitStructure;  																			//Inicjalizacja struktury konfiguracji przerwa EXTI

  GPIO_EXTILineConfig(GPIO_PortSourceGPIOC, GPIO_PinSource7);									//Aktywowanie funkcji EXTI pinw joystick'a
  GPIO_EXTILineConfig(GPIO_PortSourceGPIOC, GPIO_PinSource8);
  GPIO_EXTILineConfig(GPIO_PortSourceGPIOC, GPIO_PinSource9);
  GPIO_EXTILineConfig(GPIO_PortSourceGPIOC, GPIO_PinSource10);
  GPIO_EXTILineConfig(GPIO_PortSourceGPIOC, GPIO_PinSource11);

  EXTI_InitStructure.EXTI_Line = EXTI_Line7 | EXTI_Line8 | EXTI_Line9 | EXTI_Line10 | EXTI_Line11;
  EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling; 										//Przerwanie przy zboczu opadajcym

  EXTI_InitStructure.EXTI_LineCmd = ENABLE;																		//Uaktywnienie kontrolera EXTI
  EXTI_Init(&EXTI_InitStructure);
}

//Funkcja konfigurujca kontroler przerwa NVIC

void NVIC_Configuration(void)
{
  NVIC_InitTypeDef NVIC_InitStructure; 																				//Inicjalizacja struktury konfiguracji kontrolera przerwa

  NVIC_PriorityGroupConfig(NVIC_PriorityGroup_1); 														//Wybr grupy priorytetw

  NVIC_InitStructure.NVIC_IRQChannel = EXTI9_5_IRQn;													//Konfiguracja przerwania od GPIO piny od 5 do 9
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

  NVIC_InitStructure.NVIC_IRQChannel = EXTI15_10_IRQn; 												//Konfiguracja przerwania od GPIO piny od 10 do 15
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);
}

