/**
  ******************************************************************************
  * @file    Project/STM32F10x_StdPeriph_Template/stm32f10x_it.c 
  * @author  MCD Application Team
  * @version V3.5.0
  * @date    08-April-2011
  * @brief   Main Interrupt Service Routines.
  *          This file provides template for all exceptions handler and 
  *          peripherals interrupt service routine.
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2011 STMicroelectronics</center></h2>
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "stm32f10x_it.h"

#define INTRO 1
#define GRA 2
#define GAMEOVER 3
#define PAUZA 4

/** @addtogroup STM32F10x_StdPeriph_Template
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/******************************************************************************/
/*            Cortex-M3 Processor Exceptions Handlers                         */
/******************************************************************************/

/**
  * @brief  This function handles NMI exception.
  * @param  None
  * @retval None
  */
void NMI_Handler(void)
{
}

/**
  * @brief  This function handles Hard Fault exception.
  * @param  None
  * @retval None
  */
void HardFault_Handler(void)
{
  /* Go to infinite loop when Hard Fault exception occurs */
  while (1)
  {
  }
}

/**
  * @brief  This function handles Memory Manage exception.
  * @param  None
  * @retval None
  */
void MemManage_Handler(void)
{
  /* Go to infinite loop when Memory Manage exception occurs */
  while (1)
  {
  }
}

/**
  * @brief  This function handles Bus Fault exception.
  * @param  None
  * @retval None
  */
void BusFault_Handler(void)
{
  /* Go to infinite loop when Bus Fault exception occurs */
  while (1)
  {
  }
}

/**
  * @brief  This function handles Usage Fault exception.
  * @param  None
  * @retval None
  */
void UsageFault_Handler(void)
{
  /* Go to infinite loop when Usage Fault exception occurs */
  while (1)
  {
  }
}

/**
  * @brief  This function handles SVCall exception.
  * @param  None
  * @retval None
  */
void SVC_Handler(void)
{
}

/**
  * @brief  This function handles Debug Monitor exception.
  * @param  None
  * @retval None
  */
void DebugMon_Handler(void)
{
}

/**
  * @brief  This function handles PendSVC exception.
  * @param  None
  * @retval None
  */
void PendSV_Handler(void)
{
}

/**
  * @brief  This function handles SysTick Handler.
  * @param  None
  * @retval None
  */
void SysTick_Handler(void)
{
  extern unsigned char rkloceknr;       //zmienne zainicjalizowane w main.c
  extern unsigned char stan_gry;
  extern unsigned char introbgoff;

  rkloceknr++;                                          //losowanie klockw
  if(rkloceknr>6) rkloceknr=0;

  introbgoff++;                                       //zwikszenie zmiennej przesunicia ta
  if(introbgoff>=126) introbgoff=0;

  if(TimerGry()==1)                                   //Funkcja SpeedTimer zwraca 1 co ustalon ilo wywoa
  {                                                     //czsto wywoa reguluje zmienna speed
    if(stan_gry==GRA) KlocekDol();                         //Jeeli trwa gra to opu klocek                                          //Odwierz ekran
  }
  OdswierzEkran();
}

//Obsuga przerwania od przyciskw joystick'a, linie: GPIOE.8 i GPIOE.9
void EXTI9_5_IRQHandler(void)
{
  extern unsigned char stan_gry;
  extern unsigned int punkty,linie;
  extern unsigned char stan_gry;
  //Obsuga przerwania z GPIOC.7 - przycisk OK
  if(EXTI_GetITStatus(EXTI_Line7) != RESET)		//sprawd z ktrego pinu pochodzi przerwanie
  {
    switch(stan_gry)
    {
      case GRA:              //Jeeli gra wczona
        stan_gry=PAUZA;         //wcz pauz
      break;

      case PAUZA:               //Jeeli gra w czasie pauzy
        stan_gry=GRA;        //wr do gry
	    break;

      case INTRO:               //Jeeli wczone intro
      	stan_gry=GRA;
      	break;

      case GAMEOVER:            //Jeeli nastpia przegrana
        CzyscPlansze();         //wyczy plansz
        KlocekNowy();           //zmie klocek na nowy
        punkty=0;               //wyzeruj licznik punktw
        linie=0;                //i zaliczonych linii

        stan_gry=INTRO;         //przejd do intro
        break;
    }
    OdswierzEkran();
    EXTI_ClearITPendingBit(EXTI_Line7);         //zeruj rdo przerwania
  }

  //Obsuga przerwania z GPIOC.8 - przycisk w gr
  if(EXTI_GetITStatus(EXTI_Line8) != RESET)			//sprawd z ktrego pinu pochodzi przerwanie
  {
    if(stan_gry==GRA)						//Jeeli gra wczona
    {
      KlocekObroc();									//obr klocek
    }
    OdswierzEkran();
    EXTI_ClearITPendingBit(EXTI_Line8);					//zeruj rdo przerwania
  }

  //Obsuga przerwania z GPIOC.9 - przycisk w d
  if(EXTI_GetITStatus(EXTI_Line9) != RESET)					//sprawd z ktrego pinu pochodzi przerwanie
  {
    if(stan_gry==GRA)									//Jeeli gra wczona
    {
      KlocekDol();										//przesu klocek w d
    }
    OdswierzEkran();
    EXTI_ClearITPendingBit(EXTI_Line9);				//zeruj rdo przerwania
  }
}

//Obsuga przerwania od przyciskw joystick'a, linie: GPIOE.10, GPIOE.11 i GPIOE.12
void EXTI15_10_IRQHandler(void)
{
  extern uint8_t stan_gry;

  //Obsuga przerwania z GPIOC.10 - przycisk w prawo
  if(EXTI_GetITStatus(EXTI_Line10) != RESET)			//sprawd z ktrego pinu pochodzi przerwanie
  {
    if(stan_gry==GRA)						//Jeeli gra wczona
    {
      KlocekPrawo();								//przesu klocek w prawo
    }
    OdswierzEkran();
    EXTI_ClearITPendingBit(EXTI_Line10);				//zeruj rdo przerwania
  }
  //Obsuga przerwania z GPIOC.11 - przycisk w lewo
  else if(EXTI_GetITStatus(EXTI_Line11) != RESET)			//sprawd z ktrego pinu pochodzi przerwanie
  {
    if(stan_gry==GRA)						//Jeeli gra wczona
    {
      KlocekLewo();								//przesu klocek w lewo
    }
    OdswierzEkran();
    EXTI_ClearITPendingBit(EXTI_Line11);				//zeruj rdo przerwania
  }
}

/******************************************************************************/
/*                 STM32F10x Peripherals Interrupt Handlers                   */
/*  Add here the Interrupt Handler for the used peripheral(s) (PPP), for the  */
/*  available peripheral interrupt handler's name please refer to the startup */
/*  file (startup_stm32f10x_xx.s).                                            */
/******************************************************************************/

/**
  * @brief  This function handles PPP interrupt request.
  * @param  None
  * @retval None
  */
/*void PPP_IRQHandler(void)
{
}*/

/**
  * @}
  */ 


/******************* (C) COPYRIGHT 2011 STMicroelectronics *****END OF FILE****/
